/****************************************************************************
*  pvm.h
*
*  This include module is used by PVM control routines in the PVM'd PovRAY.
*
*  This file was written by Brad Kline.  April 1994.
*                modified by Andreas Dilger,  Feb 1995.
*                modified by Harald Deischinger, 1996 - April 1997
*                modified by Jakob Flierl, May 1999
* 
*  All the defines that need modification are at the top.
*
*  The author disclaims all warranties with regard to this software, 
*  including all implied warranties of merchant-ability and fitness. 
*  The code is simply distributed as it is.
*
*
*****************************************************************************/

/* If setpriority() doesn't exist on your system, you should change 
 * "your_pvm_arch" on the following line to whatever your actual PVM_ARCH
 * is (ie ALPHA or SCO), and please email me to tell me about it, so I can
 * add it in permanently.
 */

#if defined(LINUX)
#include <features.h>
#endif

#if defined(SUN4SOL2) || defined(FREEBSD) || defined(LINUXALPHA) || defined(your_pvm_arch)

#define NICE(x) nice(x)
extern int nice PARAMS((int prio));

#else

/* This should work for the majority of systems */

#define NICE(x) setpriority(PRIO_PROCESS,0,getpriority(PRIO_PROCESS,0)+(x))

#if !(defined __GLIBC__ && __GLIBC__ >= 2)
extern int setpriority PARAMS((int which, int who, int prio));
extern int getpriority PARAMS((int which, int who));
#endif
#endif

/* How nice to be by default. 0 is the regular job priority, while is 20 very
 * nice.  The nice value can be changed on the command line, so there is
 * probably little reason to change it here.
 */
#define PVM_DEFAULT_NICE 5

/* On BSD systems, use getwd instead of getcwd.  Change "your_pvm_arch" on
 * the following line to whatever your actual PVM_ARCH is (ie MIPS or CRAY),
 * and please email me to tell me about it, so I can add it in permanently.
 */
#if defined(NEXT) || defined(your_pvm_arch)

#define GETCWD(path) getwd(path)
extern char *getwd PARAMS((char *));

#else

#define GETCWD(path) getcwd(path,PATH_MAX+1)
extern char *getcwd PARAMS((char *, size_t ));

#endif

extern int  chdir PARAMS((const char *path));

/* Grid decomposition defaults */
#define PVM_DEFAULT_GRID_HEIGHT     32
#define PVM_DEFAULT_GRID_WIDTH      32

/* what type of encoding to use whan sending pixels */
#define pvm_upkCOL pvm_upkfloat
#define pvm_pkCOL pvm_pkfloat
#define pvm_upkSTAT pvm_upklong
#define pvm_pkSTAT pvm_pklong


/* The maximum path length. _POSIX_PATH_MAX_ = 255 in my limits.h file, but
 * not all systems have this defined, and my base SUN4 install didn't have
 * PATH_MAX in <limits.h>.*/
#ifndef PATH_MAX
#ifdef _POSIX_PATH_MAX
#define PATH_MAX _POSIX_PATH_MAX
#else  /* _POSIX_PATH_MAX doesn't exist.  A safe guess. */
#define PATH_MAX 1024
#endif  /* _POSIX_PATH_MAX */ 
#endif  /* PATH_MAX */ 

/* Structure to hold info about each slave process */
typedef struct {
    char *name;
    int pixels_done;		/* Number of pixels completed */
    int pixels_late;		/* Number of pixels finished after another slave did it */
    int block_reassigned;	/* Block number reassigned to this slave */
    int frame_assigned;		/* assigned to that frame */
    int frame_reassigned;	/* reassigned to that frame */
} pvm_slave_stat;
#define PVM_SLAVE_FREE           -1

/* Structure to hold info about each grid section */
typedef struct {
    int status;          /* Current status of block */
    int wr_line;         /* Line to be written next */
    int assigned_tid;    /* Tid of the machine assigned to this block */
    int reassigned_tid;  /* Tid of the machine reassigned to this block */
} pvm_block_stat;
/* Status indicators */
#define PVM_GRID_POINT_FREE       0
#define PVM_GRID_POINT_ASSIGNED   1
#define PVM_GRID_POINT_REASSIGNED 2
#define PVM_GRID_POINT_DONE       3
#define PVM_GRID_POINT_WRITTEN    4


/* Structure to hold info about each frame */
typedef struct {
    int status;
    COLOUR ** row_ptr;
    int row_to_write;
    int row_to_malloc;
    int block_to_assign;
    int block_to_reassign;
    int block_to_write;
    COUNTER stats[MaxStat];		/* PovRay Status */
    pvm_block_stat * block_stat;
    FILE_HANDLE* output_file;
} pvm_frame_stat;
#define PVM_FRAME_FREE		0
#define PVM_FRAME_WORKING	1
#define PVM_FRAME_DONE		2
#define PVM_FRAME_DONE_SHELLOUT	3

/* Define some message IDs to be used */
#define PVM_INIT_SLAVE         1
#define PVM_STOP_SLAVE         2
#define PVM_KILL_SLAVE         3
#define PVM_NEED_WORK          4
#define PVM_SLAVE_EXIT         5
#define PVM_SLAVE_STATS        6
#define PVM_SLAVE_RESULTS      7
#define PVM_WORK               8
#define PVM_POLL_SLAVE         9
#define PVM_ERROR_REPORT       10

/* Prototypes for public functions defined in pvm.c */
void Pvm_Master_Init PARAMS((int argc, char **argv));
void Pvm_Master_Control PARAMS((void));
void PvmMasterEnd PARAMS((void));
void Pvm_Write_Line PARAMS((FILE_HANDLE *handle, COLOUR *line, int row));
int  Pvm_Slave_Init PARAMS((void));
int  Pvm_Slave_Control PARAMS((void));
void PvmReportError PARAMS((char *vsbuffer));
void Pvm_Slave_Exit PARAMS((void));

extern char     PvmArch[];
extern int      PvmChunkWidth;
extern int      PvmChunkHeight;
extern int      PvmSlave;
extern int      PvmTasks;
extern int      PvmNice;
extern char *	PvmSlavename;
extern int	PvmNextFrame;
extern int	PvmBlockNum;
extern char	PvmWorkingDir[];
extern char **  PvmHosts;
extern int	PvmHostsN;
extern int      PvmMTid;
extern int    * tids;
